@@ -16,7 +16,7 @@ import sys
 
 from fabric import api # For checking callables against the API
 from fabric.contrib import console, files, project # Ditto
-from fabric.network import denormalize, normalize
+from fabric.network import denormalize, interpret_host_string, disconnect_all
 from fabric import state # For easily-mockable access to roles, env and etc
 from fabric.state import commands, connections, env_options
 from fabric.utils import abort, indent
@@ -101,7 +101,7 @@ def is_task(tup):
     )
 
 
-def load_fabfile(path, importer=None):
+def load_fabfile(path):
     """
     Import given fabfile path and return (docstring, callables).
 
@@ -109,8 +109,6 @@ def load_fabfile(path, importer=None):
     dictionary of ``{'name': callable}`` containing all callables which pass
     the "is a Fabric task" test.
     """
-    if importer is None:
-        importer = __import__
     # Get directory and fabfile name
     directory, fabfile = os.path.split(path)
     # If the directory isn't in the PYTHONPATH, add it so our import will work
@@ -131,7 +129,7 @@ def load_fabfile(path, importer=None):
             sys.path.insert(0, directory)
             del sys.path[i + 1]
     # Perform the import (trimming off the .py)
-    imported = importer(os.path.splitext(fabfile)[0])
+    imported = __import__(os.path.splitext(fabfile)[0])
     # Remove directory from path if we added it ourselves (just to be neat)
     if added_to_path:
         del sys.path[0]
@@ -363,14 +361,7 @@ def _merge(hosts, roles):
             value = value()
         role_hosts += value
     # Return deduped combo of hosts and role_hosts
-    return list(set(_clean_hosts(hosts + role_hosts)))
-
-
-def _clean_hosts(host_list):
-    """
-    Clean host strings to ensure no trailing whitespace, etc.
-    """
-    return [host.strip() for host in host_list]
+    return list(set(hosts + role_hosts))
 
 
 def get_hosts(command, cli_hosts, cli_roles):
@@ -535,13 +526,10 @@ def main():
                 command, cli_hosts, cli_roles)
             # If hosts found, execute the function on each host in turn
             for host in hosts:
-                username, hostname, port = normalize(host)
-                state.env.host_string = host
-                state.env.host = hostname
                 # Preserve user
                 prev_user = state.env.user
-                state.env.user = username
-                state.env.port = port
+                # Split host string and apply to env dict
+                username, hostname, port = interpret_host_string(host)
                 # Log to stdout
                 if state.output.running:
                     print("[%s] Executing task '%s'" % (host, name))
@@ -567,11 +555,5 @@ def main():
         # we might leave stale threads if we don't explicitly exit()
         sys.exit(1)
     finally:
-        # Explicitly disconnect from all servers
-        for key in connections.keys():
-            if state.output.status:
-                print "Disconnecting from %s..." % denormalize(key),
-            connections[key].close()
-            if state.output.status:
-                print "done."
+        disconnect_all()
     sys.exit(0)
